/***************************************************************************

  TCalcMeanFieldsLEM.h

  Author: Bastian M. Wojek

***************************************************************************/

/***************************************************************************
 *   Copyright (C) 2009 by Bastian M. Wojek                                *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _TCalcMeanFieldsLEM_H_
#define _TCalcMeanFieldsLEM_H_

#include <memory>

#include "TLondon1D.h"

class TMeanFieldsForScHalfSpace : public PUserFcnBase {

public:
  // default constructor
  TMeanFieldsForScHalfSpace();

  double operator()(double, const std::vector<double>&) const;
  double CalcMeanB (double, const TLondon1D_HS&) const;

private:
  std::unique_ptr<TTrimSPData> fImpProfile;

  ClassDef(TMeanFieldsForScHalfSpace,1)
};

class TMeanFieldsForScSingleLayer : public PUserFcnBase {

public:
  // default constructor
  TMeanFieldsForScSingleLayer();

  double operator()(double, const std::vector<double>&) const;
  double CalcMeanB (double, const std::vector<double>&, const std::vector<double>&, const TLondon1D_1L&) const;

private:
  std::unique_ptr<TTrimSPData> fImpProfile;

  ClassDef(TMeanFieldsForScSingleLayer,1)
};

class TMeanFieldsForScBilayer : public PUserFcnBase {

public:
  // default constructor
  TMeanFieldsForScBilayer();

  double operator()(double, const std::vector<double>&) const;
  double CalcMeanB (double, const std::vector<double>&, const std::vector<double>&, const TLondon1D_2L&, double) const;

private:
  std::unique_ptr<TTrimSPData> fImpProfile;

  ClassDef(TMeanFieldsForScBilayer,1)
};

class TMeanFieldsForScTrilayer : public PUserFcnBase {

public:
  // default constructor
  TMeanFieldsForScTrilayer();

  double operator()(double, const std::vector<double>&) const;
  double CalcMeanB (double, const std::vector<double>&, const std::vector<double>&, const TLondon1D_3L&) const;

private:
  std::unique_ptr<TTrimSPData> fImpProfile;

  ClassDef(TMeanFieldsForScTrilayer,1)
};

class TMeanFieldsForScTrilayerWithInsulator : public PUserFcnBase {

public:
  // default constructor
  TMeanFieldsForScTrilayerWithInsulator();

  double operator()(double, const std::vector<double>&) const;
  double CalcMeanB (double, const std::vector<double>&, const std::vector<double>&, const TLondon1D_3LwInsulator&) const;

private:
  std::unique_ptr<TTrimSPData> fImpProfile;

  ClassDef(TMeanFieldsForScTrilayerWithInsulator,1)
};

#endif /* _TCalcMeanFieldsLEM_H_ */
